@@ -34,11 +34,14 @@ __all__ = (
 
 __metaclass__ = type
 
+import logging
 import re
 import sys
 import urllib
 import webob
 
+log = logging.getLogger(__name__)
+
 bbbbad_errors = KeyboardInterrupt, SystemExit, MemoryError
 
 _default_content_type = 'text/html; charset=UTF-8'
@@ -183,23 +186,28 @@ class Application:
 
     def bobo_response(self, request, path, method):
         try:
+            allowed = set()
             for handler in self.handlers:
-                response = handler(request, path, method)
+                try:
+                    response = handler(request, path, method)
+                except MethodNotAllowed, exc:
+                    allowed.update(exc.allowed)
+                    continue
                 if response is not None:
                     return response
+            if allowed:
+                return self.method_not_allowed(request, method, allowed)
             return self.not_found(request, method)
         except BoboException, exc:
             return self.build_response(request, method, exc)
-        except MethodNotAllowed, v:
-            return self.method_not_allowed(request, method, v.allowed)
         except MissingFormVariable, v:
             return self.missing_form_variable(request, method, v.name)
-        except NotFound, v:
+        except NotFound:
             return self.not_found(request, method)
         except bbbbad_errors:
             raise
         except Exception:
-            if not hasattr(self, 'exception'):
+            if request.environ.get("x-wsgiorg.throw_errors"):
                 raise
             return self.exception(request, method, sys.exc_info())
 
@@ -240,8 +248,8 @@ class Application:
 
         content_type = data.content_type
         response = webob.Response(status=data.status,
-                                  headerlist=data.headers,
-                                  content_type=content_type)
+                                  headerlist=data.headers)
+        response.content_type = content_type
 
         if method == 'HEAD':
             return response
@@ -250,12 +258,7 @@ class Application:
         if isinstance(body, str):
             response.body = body
         elif _json_content_type(content_type):
-            try:
-                import json
-            except ImportError:
-                import simplejson
-                sys.modules['json'] = simplejson
-                json = simplejson
+            import json
             response.body = json.dumps(body)
         elif isinstance(body, unicode):
             response.unicode_body = body
@@ -280,6 +283,13 @@ class Application:
             "Method Not Allowed", "Invalid request method: %s" % method,
             [('Allow', ', '.join(sorted(methods)))])
 
+    def exception(self, request, method, exc_info):
+        log.exception(request.url)
+        return _err_response(
+            500, method,
+            "Internal Server Error", "An error occurred.")
+
+
 def _err_response(status, method, title, message, headers=()):
     response = webob.Response(status=status, headerlist=headers or [])
     response.content_type = 'text/html; charset=UTF-8'
@@ -1237,10 +1247,17 @@ def scan_class(class_):
         handlers.append(_make_br_method_for_name(name))
 
     def bobo_response(self, request, path, method):
+        allowed = set()
         for handler in handlers:
-            found = handler(self, request, path, method)
+            try:
+                found = handler(self, request, path, method)
+            except MethodNotAllowed, exc:
+                allowed.update(exc.allowed)
+                continue
             if found is not None:
                 return found
+        if allowed:
+            raise MethodNotAllowed(allowed)
 
     old = class_.__dict__.get('bobo_response')
     if isinstance(old, _subroute_class_method):
