@@ -34,6 +34,8 @@ import traceback
 import types
 import webob
 
+mimetypes.init()
+
 def run_server(app, port):
     wsgiref.simple_server.make_server('', port, app).serve_forever()
 
@@ -59,7 +61,7 @@ class Directory:
     @bobo.query('/')
     def index(self):
         links = []
-        for name in os.listdir(self.path):
+        for name in sorted(os.listdir(self.path)):
             if os.path.isdir(os.path.join(self.path, name)):
                 name += '/'
             links.append('<a href="%s">%s</a>' % (name, name))
@@ -69,7 +71,7 @@ class Directory:
           %s
         </body>
         </html>
-        """ % (self.path[len(self.root):], '<br>\n  '.join(links))
+        """ % (self.path[len(self.root):], '<br>\n          '.join(links))
 
     @bobo.subroute('/:name')
     def traverse(self, request, name):
